#!/usr/bin/perl -w
# A script to pick up the cDNA record from cDNA database according to the pep record.
#    perl cDNApicker.pl pep.fas cDNA.fas

use strict;
use warnings;

my $keyWordBeforeCDNAid = "transcript:";    ### Pick up the cDNA id form pep file name line.
my $cDNAid = "";

my $AAseqFile  = $ARGV[0];
my $db         = $ARGV[1];
my $outNucFile = "Out_".$db;
my $countCDNAprint = 0;


#### Read the amino acid file
open(AASEQ,"$AAseqFile") or die"$!";
my @aaSeqFile = <AASEQ>;
close AASEQ;


### Push the AA seq fasta formats into the hash and array
my ($ref_fasHashAA,$ref_nameArrayAA, $seqLengthAA) = &fasHashReader(\@aaSeqFile);

### Check the name line of fasta format.  If the name line is not the Ensembl format, the program is quitted.
foreach (@$ref_nameArrayAA) {
  if($_ =~ /transcript:[a-zA-Z]/){
    next;
  } else {
    print "The name line of $AAseqFile is not Ensembl format\n";
    print "  $_\n";
    print "The name line should be written as\n";
    print "  >ENSP00000374981 pep:known chromosome:GRCh37:14:106053226:106054732:-1 gene:ENSG00000211890 transcript:ENST00000390539\n";
    print "program stopped.\n";
    exit;
  }
}


##### Main program

open(OUTA,">$outNucFile");
foreach my $aaName (@$ref_nameArrayAA){
  $aaName =~ /transcript:(\w+)$/;                           # Pick up the ENST ID from the name line of AA file
  $cDNAid = $1;
  my $refhitRecordHash = &seqRetFromFastaDBwithWhile($cDNAid,$db);
  my %hitRecord = %$refhitRecordHash;
  foreach my $cdnaNameLine (keys %hitRecord) {
#    print "  $cDNAid -> $cdnaNameLine\n";
    print OUTA "$cdnaNameLine\n$hitRecord{$cdnaNameLine}\n";
    $countCDNAprint++;
  }
}
#print "\n";
close OUTA;

print "  pep : ".@$ref_nameArrayAA." records\n";
print "  cdna : $countCDNAprint found\n";

exit;


##############
sub fasHashReader{

  my $name = "";
  my $seq = "";
  my $seqLength = "";
  my $flag = 0;
  my @nameArraySR = ( );
  my %fasHashSR = ( );

  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine =~ /^>/){  
      if($flag == 0){
        $name = $inFileLine;
        $flag = 1;  
      } else {
        $fasHashSR{$name} = $seq;  
        push(@nameArraySR,$name);
        $name = "$inFileLine";
        $seq  = "";
      }
    } else {
      $seq .= $inFileLine; 
    }
  }
  $fasHashSR{$name} = $seq;
  $seqLength = length($seq);
  push(@nameArraySR,$name);
  return (\%fasHashSR,\@nameArraySR,\$seqLength);
}


###########
# This subroutine uses "while", so it assumes one records for one ID in the DB.
sub seqRetFromFastaDBwithWhile {

  my ($cDNAid,$dbName) = @_;

  my $flag = 0;
  my $key  = "";                # key of the above hash
  my $seq  = "";                # seq of the above hash
  my %hitRecord = ( );         # Hit record. 

  open(INFILEWH,"$dbName") or die "$!";
  while (my $dbLine = <INFILEWH>) {
    chomp($dbLine);
    if($flag == 1){
      if($dbLine =~ /^>/){
        last;
      } else {
        $seq .= $dbLine;
      }
    }
    if($dbLine =~ /^>/ && $dbLine =~ /$cDNAid/ ){
      $key = $dbLine;
      $flag = 1;
    }
  }
  $hitRecord{$key} = $seq;

  close INFILEWH;
  return \%hitRecord;
}